use egui_winit_vulkano::egui::ahash::HashMap;
use winit::{
    event::{ElementState, KeyEvent},
    keyboard::PhysicalKey,
};

use super::virtual_input::VirtualInput;

#[derive(Debug, Default)]
pub struct KeyboardState {
    key_states: HashMap<PhysicalKey, ElementState>,
}

impl KeyboardState {
    pub fn get_key_states(&self) -> &HashMap<PhysicalKey, ElementState> {
        &self.key_states
    }

    pub fn get_key_state(&self, key: PhysicalKey) -> &ElementState {
        self.key_states.get(&key).unwrap_or(&ElementState::Released)
    }

    pub fn process_window_event(&mut self, event: &KeyEvent, virtual_input: &mut VirtualInput) {
        let key_state = self.key_states.get(&event.physical_key);
        let new_key_state = match key_state {
            Some(key_state) => match event.state {
                ElementState::Pressed => match key_state {
                    ElementState::Released => Some(ElementState::Pressed),
                    ElementState::Pressed => None,
                },
                ElementState::Released => match key_state {
                    ElementState::Released => None,
                    ElementState::Pressed => Some(ElementState::Released),
                },
            },
            None => match event.state {
                ElementState::Pressed => Some(ElementState::Pressed),
                ElementState::Released => Some(ElementState::Released),
            },
        };
        if let Some(new_key_state) = new_key_state {
            self.key_states.insert(event.physical_key, new_key_state);
            virtual_input.update_key_binding(event.physical_key, new_key_state);
        }
    }
}
