use winit::{event::ElementState, keyboard::PhysicalKey};

use super::virtual_binding::VirtualBinding;

pub struct VirtualBindingState {
    pub value: f32,
    pub binding: VirtualBinding,
}

pub struct VirtualInputState {
    pub value: f32,
    pub bindings: Vec<VirtualBindingState>,
}

impl VirtualInputState {
    pub fn update_from_key(&mut self, key: PhysicalKey, key_state: ElementState) {
        let mut new_value = 0.0;
        for binding in &mut self.bindings {
            if let VirtualBinding::Keyboard(binding_key, direction) = &binding.binding {
                if binding_key == &key {
                    if key_state == ElementState::Pressed {
                        binding.value += f32::from(direction);
                    } else {
                        binding.value = 0.0;
                    }
                }
            }
            new_value += binding.value;
        }
        self.value = new_value;
    }

    pub fn update_from_mouse(&mut self, delta: &glam::Vec2) {
        let mut new_value = 0.0;
        for binding in &mut self.bindings {
            match &binding.binding {
                VirtualBinding::MouseX(direction) => {
                    binding.value = f32::from(direction) * delta.x;
                }
                VirtualBinding::MouseY(direction) => {
                    binding.value = f32::from(direction) * delta.y;
                }
                _ => {}
            }
            new_value += binding.value;
        }
        self.value = new_value;
    }
}
